@@ -2,10 +2,10 @@
 /*
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
-#include <stdio.h>
+#include "libbb.h"
 #include "unarchive.h"
 
-void header_list(const file_header_t *file_header)
+void FAST_FUNC header_list(const file_header_t *file_header)
 {
 	puts(file_header->name);
 }
