@@ -2,37 +2,37 @@
 /*
  * Mini kbd_mode implementation for busybox
  *
- * Copyright (C) 2007 Loc Greni <loic.grenie@gmail.com>
+ * Copyright (C) 2007 Loic Grenie <loic.grenie@gmail.com>
  *   written using Andries Brouwer <aeb@cwi.nl>'s kbd_mode from
  *   console-utils v0.2.3, licensed under GNU GPLv2
  *
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
- *
  */
-
 #include "libbb.h"
 #include <linux/kd.h>
 
 int kbd_mode_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int kbd_mode_main(int argc UNUSED_PARAM, char **argv)
 {
-	int fd;
-	unsigned opt;
 	enum {
 		SCANCODE  = (1 << 0),
 		ASCII	  = (1 << 1),
 		MEDIUMRAW = (1 << 2),
-		UNICODE	  = (1 << 3)
+		UNICODE	  = (1 << 3),
 	};
-	static const char KD_xxx[] ALIGN1 = "saku";
-	opt = getopt32(argv, KD_xxx);
-	fd = get_console_fd_or_die();
+	int fd;
+	unsigned opt;
+	const char *tty_name = CURRENT_TTY;
+
+	opt = getopt32(argv, "sakuC:", &tty_name);
+	fd = xopen(tty_name, O_NONBLOCK);
+	opt &= 0xf; /* clear -C bit, see (*) */
 
 	if (!opt) { /* print current setting */
 		const char *mode = "unknown";
 		int m;
 
-		ioctl(fd, KDGKBMODE, &m);
+		xioctl(fd, KDGKBMODE, &m);
 		if (m == K_RAW)
 			mode = "raw (scancode)";
 		else if (m == K_XLATE)
@@ -43,6 +43,7 @@ int kbd_mode_main(int argc UNUSED_PARAM, char **argv)
 			mode = "Unicode (UTF-8)";
 		printf("The keyboard is in %s mode\n", mode);
 	} else {
+		/* here we depend on specific bits assigned to options (*) */
 		opt = opt & UNICODE ? 3 : opt >> 1;
 		/* double cast prevents warnings about widening conversion */
 		xioctl(fd, KDSKBMODE, (void*)(ptrdiff_t)opt);
