@@ -17,20 +17,16 @@
 
 int bb_cat(char **argv)
 {
-	static const char *const argv_dash[] = { "-", NULL };
-
 	int fd;
 	int retval = EXIT_SUCCESS;
 
 	if (!*argv)
-		argv = (char**) &argv_dash;
+		argv = (char**) &bb_argv_dash;
 
 	do {
-		fd = STDIN_FILENO;
-		if (!LONE_DASH(*argv))
-			fd = open_or_warn(*argv, O_RDONLY);
+		fd = open_or_warn_stdin(*argv);
 		if (fd >= 0) {
-			/* This is not an xfunc - never exits */
+			/* This is not a xfunc - never exits */
 			off_t r = bb_copyfd_eof(fd, STDOUT_FILENO);
 			if (fd != STDIN_FILENO)
 				close(fd);
@@ -43,8 +39,8 @@ int bb_cat(char **argv)
 	return retval;
 }
 
-int cat_main(int argc, char **argv);
-int cat_main(int argc, char **argv)
+int cat_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
+int cat_main(int argc UNUSED_PARAM, char **argv)
 {
 	getopt32(argv, "u");
 	argv += optind;
