@@ -1,4 +1,7 @@
 /* vi: set sw=4 ts=4: */
+/*
+ * Licensed under GPLv2, see file LICENSE in this tarball for details.
+ */
 
 /* BB_AUDIT SUSv3 N/A -- Apparently a busybox (obsolete?) extension. */
 
@@ -6,7 +9,7 @@
 
 /* This is a NOFORK applet. Be very careful! */
 
-int length_main(int argc, char **argv);
+int length_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int length_main(int argc, char **argv)
 {
 	if ((argc != 2) || (**(++argv) == '-')) {
