@@ -13,10 +13,10 @@
 
 /* This is a NOFORK applet. Be very careful! */
 
-int usleep_main(int argc, char **argv);
-int usleep_main(int argc, char **argv)
+int usleep_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
+int usleep_main(int argc UNUSED_PARAM, char **argv)
 {
-	if (argc != 2) {
+	if (!argv[1]) {
 		bb_show_usage();
 	}
 
