@@ -2,8 +2,7 @@
 /*
  * See README for additional information
  *
- * This file can be redistributed under the terms of the GNU Library General
- * Public License
+ * Licensed under GPLv2, see file LICENSE in this tarball for details.
  */
 
 #include "libbb.h"
@@ -29,33 +28,20 @@ static void close_silently(int fd)
 
 /* Iterate a function on each entry of a directory */
 int iterate_on_dir(const char *dir_name,
-		int (*func)(const char *, struct dirent *, void *),
-		void * private)
+		int FAST_FUNC (*func)(const char *, struct dirent *, void *),
+		void *private)
 {
 	DIR *dir;
-	struct dirent *de, *dep;
-	int max_len, len;
-
-	max_len = PATH_MAX + sizeof(struct dirent);
-	de = xmalloc(max_len+1);
-	memset(de, 0, max_len+1);
+	struct dirent *de;
 
 	dir = opendir(dir_name);
 	if (dir == NULL) {
-		free(de);
 		return -1;
 	}
-	while ((dep = readdir(dir))) {
-		len = sizeof(struct dirent);
-		if (len < dep->d_reclen)
-			len = dep->d_reclen;
-		if (len > max_len)
-			len = max_len;
-		memcpy(de, dep, len);
+	while ((de = readdir(dir)) != NULL) {
 		func(dir_name, de, private);
 	}
 	closedir(dir);
-	free(de);
 	return 0;
 }
 
@@ -141,59 +127,87 @@ int fgetsetflags(const char *name, unsigned long *get_flags, unsigned long set_f
 
 
 /* Print file attributes on an ext2 file system */
-struct flags_name {
-	unsigned long	flag;
-	char		short_name;
-	const char	*long_name;
-};
-
-/* TODO: apart from I and (disabled) COMPRESSION flags, this
- * is a duplicate of a table from chattr. Merge? */
-static const struct flags_name flags_array[] = {
-	{ EXT2_SECRM_FL, 's', "Secure_Deletion" },
-	{ EXT2_UNRM_FL, 'u' , "Undelete" },
-	{ EXT2_SYNC_FL, 'S', "Synchronous_Updates" },
-	{ EXT2_DIRSYNC_FL, 'D', "Synchronous_Directory_Updates" },
-	{ EXT2_IMMUTABLE_FL, 'i', "Immutable" },
-	{ EXT2_APPEND_FL, 'a', "Append_Only" },
-	{ EXT2_NODUMP_FL, 'd', "No_Dump" },
-	{ EXT2_NOATIME_FL, 'A', "No_Atime" },
-	{ EXT2_COMPR_FL, 'c', "Compression_Requested" },
+const uint32_t e2attr_flags_value[] = {
 #ifdef ENABLE_COMPRESSION
-	{ EXT2_COMPRBLK_FL, 'B', "Compressed_File" },
-	{ EXT2_DIRTY_FL, 'Z', "Compressed_Dirty_File" },
-	{ EXT2_NOCOMPR_FL, 'X', "Compression_Raw_Access" },
-	{ EXT2_ECOMPR_FL, 'E', "Compression_Error" },
+	EXT2_COMPRBLK_FL,
+	EXT2_DIRTY_FL,
+	EXT2_NOCOMPR_FL,
+	EXT2_ECOMPR_FL,
 #endif
-	{ EXT3_JOURNAL_DATA_FL, 'j', "Journaled_Data" },
-	{ EXT2_INDEX_FL, 'I', "Indexed_directory" },
-	{ EXT2_NOTAIL_FL, 't', "No_Tailmerging" },
-	{ EXT2_TOPDIR_FL, 'T', "Top_of_Directory_Hierarchies" },
-	{ 0, '\0', NULL }
+	EXT2_INDEX_FL,
+	EXT2_SECRM_FL,
+	EXT2_UNRM_FL,
+	EXT2_SYNC_FL,
+	EXT2_DIRSYNC_FL,
+	EXT2_IMMUTABLE_FL,
+	EXT2_APPEND_FL,
+	EXT2_NODUMP_FL,
+	EXT2_NOATIME_FL,
+	EXT2_COMPR_FL,
+	EXT3_JOURNAL_DATA_FL,
+	EXT2_NOTAIL_FL,
+	EXT2_TOPDIR_FL
 };
 
-void print_flags(FILE *f, unsigned long flags, unsigned options)
-{
-	const struct flags_name *fp;
+const char e2attr_flags_sname[] =
+#ifdef ENABLE_COMPRESSION
+	"BZXE"
+#endif
+	"I"
+	"suSDiadAcjtT";
 
+static const char e2attr_flags_lname[] =
+#ifdef ENABLE_COMPRESSION
+	"Compressed_File" "\0"
+	"Compressed_Dirty_File" "\0"
+	"Compression_Raw_Access" "\0"
+	"Compression_Error" "\0"
+#endif
+	"Indexed_directory" "\0"
+	"Secure_Deletion" "\0"
+	"Undelete" "\0"
+	"Synchronous_Updates" "\0"
+	"Synchronous_Directory_Updates" "\0"
+	"Immutable" "\0"
+	"Append_Only" "\0"
+	"No_Dump" "\0"
+	"No_Atime" "\0"
+	"Compression_Requested" "\0"
+	"Journaled_Data" "\0"
+	"No_Tailmerging" "\0"
+	"Top_of_Directory_Hierarchies" "\0"
+	/* Another trailing NUL is added by compiler */;
+
+void print_e2flags(FILE *f, unsigned long flags, unsigned options)
+{
+	const uint32_t *fv;
+	const char *fn;
+
+	fv = e2attr_flags_value;
 	if (options & PFOPT_LONG) {
 		int first = 1;
-		for (fp = flags_array; fp->short_name; fp++) {
-			if (flags & fp->flag) {
+		fn = e2attr_flags_lname;
+		do {
+			if (flags & *fv) {
 				if (!first)
 					fputs(", ", f);
-				fputs(fp->long_name, f);
+				fputs(fn, f);
 				first = 0;
 			}
-		}
+			fv++;
+			fn += strlen(fn) + 1;
+		} while (*fn);
 		if (first)
 			fputs("---", f);
 	} else {
-		for (fp = flags_array; fp->short_name; fp++) {
+		fn = e2attr_flags_sname;
+		do  {
 			char c = '-';
-			if (flags & fp->flag)
-				c = fp->short_name;
+			if (flags & *fv)
+				c = *fn;
 			fputc(c, f);
-		}
+			fv++;
+			fn++;
+		} while (*fn);
 	}
 }
