@@ -55,8 +55,8 @@ static const char usage_messages[] ALIGN1 = ""
 #ifdef SINGLE_APPLET_MAIN
 #undef ENABLE_FEATURE_INDIVIDUAL
 #define ENABLE_FEATURE_INDIVIDUAL 1
-#undef USE_FEATURE_INDIVIDUAL
-#define USE_FEATURE_INDIVIDUAL(...) __VA_ARGS__
+#undef IF_FEATURE_INDIVIDUAL
+#define IF_FEATURE_INDIVIDUAL(...) __VA_ARGS__
 #endif
 
 
@@ -73,8 +73,7 @@ static const char *unpack_usage_messages(void)
 
 	i = start_bunzip(&bd,
 			/* src_fd: */ -1,
-//FIXME: can avoid storing these 2 bytes!
-			/* inbuf:  */ (void *)packed_usage + 2,
+			/* inbuf:  */ (void *)packed_usage,
 			/* len:    */ sizeof(packed_usage));
 	/* read_bunzip can longjmp to start_bunzip, and ultimately
 	 * end up here with i != 0 on read data errors! Not trivial */
@@ -99,7 +98,7 @@ static const char *unpack_usage_messages(void)
 
 static void full_write2_str(const char *str)
 {
-	full_write(STDERR_FILENO, str, strlen(str));
+	xwrite_str(STDERR_FILENO, str);
 }
 
 void FAST_FUNC bb_show_usage(void)
@@ -111,9 +110,9 @@ void FAST_FUNC bb_show_usage(void)
 		const char *usage_string = p = unpack_usage_messages();
 
 		if (*p == '\b') {
-			full_write2_str("\nNo help available.\n\n");
+			full_write2_str("No help available.\n\n");
 		} else {
-			full_write2_str("\nUsage: "SINGLE_APPLET_STR" ");
+			full_write2_str("Usage: "SINGLE_APPLET_STR" ");
 			full_write2_str(p);
 			full_write2_str("\n\n");
 		}
@@ -179,10 +178,10 @@ int FAST_FUNC find_applet_by_name(const char *name)
 
 
 void lbb_prepare(const char *applet
-		USE_FEATURE_INDIVIDUAL(, char **argv))
+		IF_FEATURE_INDIVIDUAL(, char **argv))
 				MAIN_EXTERNALLY_VISIBLE;
 void lbb_prepare(const char *applet
-		USE_FEATURE_INDIVIDUAL(, char **argv))
+		IF_FEATURE_INDIVIDUAL(, char **argv))
 {
 #ifdef __GLIBC__
 	(*(int **)&bb_errno) = __errno_location();
@@ -224,7 +223,7 @@ bool re_execed;
 /* If not built as a single-applet executable... */
 #if !defined(SINGLE_APPLET_MAIN)
 
-USE_FEATURE_SUID(static uid_t ruid;)  /* real uid */
+IF_FEATURE_SUID(static uid_t ruid;)  /* real uid */
 
 #if ENABLE_FEATURE_SUID_CONFIG
 
@@ -500,7 +499,7 @@ static void parse_config_file(void)
 #else
 static inline void parse_config_file(void)
 {
-	USE_FEATURE_SUID(ruid = getuid();)
+	IF_FEATURE_SUID(ruid = getuid();)
 }
 #endif /* FEATURE_SUID_CONFIG */
 
@@ -569,12 +568,12 @@ static void check_suid(int applet_no)
 #endif
  check_need_suid:
 #endif
-	if (APPLET_SUID(applet_no) == _BB_SUID_ALWAYS) {
+	if (APPLET_SUID(applet_no) == _BB_SUID_REQUIRE) {
 		/* Real uid is not 0. If euid isn't 0 too, suid bit
 		 * is most probably not set on our executable */
 		if (geteuid())
 			bb_error_msg_and_die("must be suid to work properly");
-	} else if (APPLET_SUID(applet_no) == _BB_SUID_NEVER) {
+	} else if (APPLET_SUID(applet_no) == _BB_SUID_DROP) {
 		xsetgid(rgid);  /* drop all privileges */
 		xsetuid(ruid);
 	}
@@ -586,7 +585,8 @@ static void check_suid(int applet_no)
 
 #if ENABLE_FEATURE_INSTALLER
 /* create (sym)links for each applet */
-static void install_links(const char *busybox, int use_symbolic_links)
+static void install_links(const char *busybox, int use_symbolic_links,
+		char *custom_install_dir)
 {
 	/* directory table
 	 * this should be consistent w/ the enum,
@@ -612,7 +612,7 @@ static void install_links(const char *busybox, int use_symbolic_links)
 
 	for (i = 0; i < ARRAY_SIZE(applet_main); i++) {
 		fpc = concat_path_file(
-				install_dir[APPLET_INSTALL_LOC(i)],
+				custom_install_dir ? custom_install_dir : install_dir[APPLET_INSTALL_LOC(i)],
 				APPLET_NAME(i));
 		// debug: bb_error_msg("%slinking %s to busybox",
 		//		use_symbolic_links ? "sym" : "", fpc);
@@ -624,7 +624,7 @@ static void install_links(const char *busybox, int use_symbolic_links)
 	}
 }
 #else
-#define install_links(x,y) ((void)0)
+#define install_links(x,y,z) ((void)0)
 #endif /* FEATURE_INSTALLER */
 
 /* If we were called as "busybox..." */
@@ -633,15 +633,14 @@ static int busybox_main(char **argv)
 	if (!argv[1]) {
 		/* Called without arguments */
 		const char *a;
-		unsigned col, output_width;
+		int col;
+		unsigned output_width;
  help:
 		output_width = 80;
 		if (ENABLE_FEATURE_AUTOWIDTH) {
 			/* Obtain the terminal width */
 			get_terminal_width_height(0, &output_width, NULL);
 		}
-		/* leading tab and room to wrap */
-		output_width -= MAX_APPLET_NAME_LEN + 8;
 
 		dup2(1, 2);
 		full_write2_str(bb_banner); /* reuse const string... */
@@ -661,29 +660,39 @@ static int busybox_main(char **argv)
 		       "Currently defined functions:\n");
 		col = 0;
 		a = applet_names;
+		/* prevent last comma to be in the very last pos */
+		output_width--;
 		while (*a) {
-			int len;
-			if (col > output_width) {
+			int len2 = strlen(a) + 2;
+			if (col >= (int)output_width - len2) {
 				full_write2_str(",\n");
 				col = 0;
 			}
-			full_write2_str(col ? ", " : "\t");
+			if (col == 0) {
+				col = 6;
+				full_write2_str("\t");
+			} else {
+				full_write2_str(", ");
+			}
 			full_write2_str(a);
-			len = strlen(a);
-			col += len + 2;
-			a += len + 1;
+			col += len2;
+			a += len2 - 1;
 		}
 		full_write2_str("\n\n");
 		return 0;
 	}
 
 	if (ENABLE_FEATURE_INSTALLER && strcmp(argv[1], "--install") == 0) {
+		int use_symbolic_links;
 		const char *busybox;
 		busybox = xmalloc_readlink(bb_busybox_exec_path);
 		if (!busybox)
 			busybox = bb_busybox_exec_path;
-		/* -s makes symlinks */
-		install_links(busybox, argv[2] && strcmp(argv[2], "-s") == 0);
+		/* busybox --install [-s] [DIR]: */
+		/* -s: make symlinks */
+		/* DIR: directory to install links to */
+		use_symbolic_links = (argv[2] && strcmp(argv[2], "-s") == 0 && argv++);
+		install_links(busybox, use_symbolic_links, argv[2]);
 		return 0;
 	}
 
@@ -754,10 +763,10 @@ int main(int argc UNUSED_PARAM, char **argv)
 #if defined(SINGLE_APPLET_MAIN)
 	/* Only one applet is selected by the user! */
 	/* applet_names in this case is just "applet\0\0" */
-	lbb_prepare(applet_names USE_FEATURE_INDIVIDUAL(, argv));
+	lbb_prepare(applet_names IF_FEATURE_INDIVIDUAL(, argv));
 	return SINGLE_APPLET_MAIN(argc, argv);
 #else
-	lbb_prepare("busybox" USE_FEATURE_INDIVIDUAL(, argv));
+	lbb_prepare("busybox" IF_FEATURE_INDIVIDUAL(, argv));
 
 #if !BB_MMU
 	/* NOMMU re-exec trick sets high-order bit in first byte of name */
