@@ -11,7 +11,7 @@
 
 /* Used from NOFORK applets. Must not allocate anything */
 
-int remove_file(const char *path, int flags)
+int FAST_FUNC remove_file(const char *path, int flags)
 {
 	struct stat path_stat;
 
