@@ -9,15 +9,15 @@
 
 #include "libbb.h"
 
-char *skip_whitespace(const char *s)
+char* FAST_FUNC skip_whitespace(const char *s)
 {
-	/* NB: isspace('0') returns 0 */
+	/* NB: isspace('\0') returns 0 */
 	while (isspace(*s)) ++s;
 
 	return (char *) s;
 }
 
-char *skip_non_whitespace(const char *s)
+char* FAST_FUNC skip_non_whitespace(const char *s)
 {
 	while (*s && !isspace(*s)) ++s;
 
