@@ -10,18 +10,22 @@
 
 #include "libbb.h"
 
-void trim(char *s)
+void FAST_FUNC trim(char *s)
 {
 	size_t len = strlen(s);
 	size_t lws;
 
 	/* trim trailing whitespace */
-	while (len && isspace(s[len-1])) --len;
+	while (len && isspace(s[len-1]))
+		--len;
 
 	/* trim leading whitespace */
 	if (len) {
 		lws = strspn(s, " \n\r\t\v");
-		memmove(s, s + lws, len -= lws);
+		if (lws) {
+			len -= lws;
+			memmove(s, s + lws, len);
+		}
 	}
 	s[len] = '\0';
 }
