@@ -19,7 +19,9 @@ enum { STACK_SIZE = (COMMON_BUFSIZE - offsetof(struct globals, stack)) / sizeof(
 #define pointer   (G.pointer   )
 #define base      (G.base      )
 #define stack     (G.stack     )
-#define INIT_G() do { } while (0)
+#define INIT_G() do { \
+	base = 10; \
+} while (0)
 
 
 static void push(double a)
@@ -53,12 +55,14 @@ static void mul(void)
 	push(pop() * pop());
 }
 
+#if ENABLE_FEATURE_DC_LIBM
 static void power(void)
 {
 	double topower = pop();
 
 	push(pow(pop(), topower));
 }
+#endif
 
 static void divide(void)
 {
@@ -96,19 +100,45 @@ static void not(void)
 
 static void set_output_base(void)
 {
-	base = (unsigned)pop();
-	if ((base != 10) && (base != 16)) {
-		bb_error_msg("error, base %d is not supported", base);
+	static const char bases[] ALIGN1 = { 2, 8, 10, 16, 0 };
+	unsigned b = (unsigned)pop();
+
+	base = *strchrnul(bases, b);
+	if (base == 0) {
+		bb_error_msg("error, base %u is not supported", b);
 		base = 10;
 	}
 }
 
 static void print_base(double print)
 {
-	if (base == 16)
-		printf("%x\n", (unsigned)print);
-	else
+	unsigned x, i;
+
+	if (base == 10) {
 		printf("%g\n", print);
+		return;
+	}
+
+	x = (unsigned)print;
+	switch (base) {
+	case 16:
+		printf("%x\n", x);
+		break;
+	case 8:
+		printf("%o\n", x);
+		break;
+	default: /* base 2 */
+		i = (unsigned)INT_MAX + 1;
+		do {
+			if (x & i) break;
+			i >>= 1;
+		} while (i > 1);
+		do {
+			bb_putchar('1' - !(x & i));
+			i >>= 1;
+		} while (i);
+		bb_putchar('\n');
+	}
 }
 
 static void print_stack_no_pop(void)
@@ -137,9 +167,11 @@ static const struct op operators[] = {
 	{"mul", mul},
 	{"/",   divide},
 	{"div", divide},
+#if ENABLE_FEATURE_DC_LIBM
 	{"**",  power},
 	{"exp", power},
 	{"pow", power},
+#endif
 	{"%",   mod},
 	{"mod", mod},
 	{"and", and},
