@@ -2,7 +2,7 @@
 /*
  * raidautorun implementation for busybox
  *
- * Copyright (C) 2006 Bernhard Fischer
+ * Copyright (C) 2006 Bernhard Reutner-Fischer
  *
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  *
@@ -13,7 +13,7 @@
 #include <linux/major.h>
 #include <linux/raid/md_u.h>
 
-int raidautorun_main(int argc, char **argv);
+int raidautorun_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int raidautorun_main(int argc, char **argv)
 {
 	if (argc != 2)
