@@ -2,7 +2,7 @@
 /*
  * strings implementation for busybox
  *
- * Copyright Tito Ragusa <farmatito@tiscali.it>
+ * Copyright 2003 Tito Ragusa <farmatito@tiscali.it>
  *
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
@@ -18,7 +18,6 @@ int strings_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int strings_main(int argc UNUSED_PARAM, char **argv)
 {
 	int n, c, status = EXIT_SUCCESS;
-	unsigned opt;
 	unsigned count;
 	off_t offset;
 	FILE *file;
@@ -26,7 +25,7 @@ int strings_main(int argc UNUSED_PARAM, char **argv)
 	const char *fmt = "%s: ";
 	const char *n_arg = "4";
 
-	opt = getopt32(argv, "afon:", &n_arg);
+	getopt32(argv, "afon:", &n_arg);
 	/* -a is our default behaviour */
 	/*argc -= optind;*/
 	argv += optind;
@@ -56,10 +55,10 @@ int strings_main(int argc UNUSED_PARAM, char **argv)
 				} else {
 					string[count] = c;
 					if (count == n) {
-						if (opt & PRINT_NAME) {
+						if (option_mask32 & PRINT_NAME) {
 							printf(fmt, *argv);
 						}
-						if (opt & PRINT_OFFSET) {
+						if (option_mask32 & PRINT_OFFSET) {
 							printf("%7"OFF_FMT"o ", offset - n);
 						}
 						fputs(string, stdout);
