@@ -8,19 +8,23 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
+/* Note that unlike older versions of modules.dep/depmod (busybox and m-i-t),
+ * we expect the full dependency list to be specified in modules.dep.
+ * Older versions would only export the direct dependency list.
+ */
 #include "libbb.h"
 #include "modutils.h"
 #include <sys/utsname.h>
 #include <fnmatch.h>
 
-//#define DBG(...) bb_error_msg(__VA_ARGS__)
+//#define DBG(fmt, ...) bb_error_msg("%s: " fmt, __func__, ## __VA_ARGS__)
 #define DBG(...) ((void)0)
 
-#define MODULE_FLAG_LOADED		0x0001
-#define MODULE_FLAG_NEED_DEPS		0x0002
+#define MODULE_FLAG_LOADED              0x0001
+#define MODULE_FLAG_NEED_DEPS           0x0002
 /* "was seen in modules.dep": */
-#define MODULE_FLAG_FOUND_IN_MODDEP	0x0004
-#define MODULE_FLAG_BLACKLISTED		0x0008
+#define MODULE_FLAG_FOUND_IN_MODDEP     0x0004
+#define MODULE_FLAG_BLACKLISTED         0x0008
 
 struct module_entry { /* I'll call it ME. */
 	unsigned flags;
@@ -33,18 +37,30 @@ struct module_entry { /* I'll call it ME. */
 	llist_t *deps; /* strings. modules we depend on */
 };
 
-#define MODPROBE_OPTS  "acdlnrt:VC:" USE_FEATURE_MODPROBE_BLACKLIST("b")
+/* NB: INSMOD_OPT_SILENT bit suppresses ONLY non-existent modules,
+ * not deleted ones (those are still listed in modules.dep).
+ * module-init-tools version 3.4:
+ * # modprobe bogus
+ * FATAL: Module bogus not found. [exitcode 1]
+ * # modprobe -q bogus            [silent, exitcode still 1]
+ * but:
+ * # rm kernel/drivers/net/dummy.ko
+ * # modprobe -q dummy
+ * FATAL: Could not open '/lib/modules/xxx/kernel/drivers/net/dummy.ko': No such file or directory
+ * [exitcode 1]
+ */
+#define MODPROBE_OPTS  "acdlnrt:VC:" IF_FEATURE_MODPROBE_BLACKLIST("b")
 enum {
-	MODPROBE_OPT_INSERT_ALL	= (INSMOD_OPT_UNUSED << 0), /* a */
-	MODPROBE_OPT_DUMP_ONLY	= (INSMOD_OPT_UNUSED << 1), /* c */
-	MODPROBE_OPT_D		= (INSMOD_OPT_UNUSED << 2), /* d */
-	MODPROBE_OPT_LIST_ONLY	= (INSMOD_OPT_UNUSED << 3), /* l */
-	MODPROBE_OPT_SHOW_ONLY	= (INSMOD_OPT_UNUSED << 4), /* n */
-	MODPROBE_OPT_REMOVE	= (INSMOD_OPT_UNUSED << 5), /* r */
-	MODPROBE_OPT_RESTRICT	= (INSMOD_OPT_UNUSED << 6), /* t */
-	MODPROBE_OPT_VERONLY	= (INSMOD_OPT_UNUSED << 7), /* V */
-	MODPROBE_OPT_CONFIGFILE	= (INSMOD_OPT_UNUSED << 8), /* C */
-	MODPROBE_OPT_BLACKLIST	= (INSMOD_OPT_UNUSED << 9) * ENABLE_FEATURE_MODPROBE_BLACKLIST,
+	MODPROBE_OPT_INSERT_ALL = (INSMOD_OPT_UNUSED << 0), /* a */
+	MODPROBE_OPT_DUMP_ONLY  = (INSMOD_OPT_UNUSED << 1), /* c */
+	MODPROBE_OPT_D          = (INSMOD_OPT_UNUSED << 2), /* d */
+	MODPROBE_OPT_LIST_ONLY  = (INSMOD_OPT_UNUSED << 3), /* l */
+	MODPROBE_OPT_SHOW_ONLY  = (INSMOD_OPT_UNUSED << 4), /* n */
+	MODPROBE_OPT_REMOVE     = (INSMOD_OPT_UNUSED << 5), /* r */
+	MODPROBE_OPT_RESTRICT   = (INSMOD_OPT_UNUSED << 6), /* t */
+	MODPROBE_OPT_VERONLY    = (INSMOD_OPT_UNUSED << 7), /* V */
+	MODPROBE_OPT_CONFIGFILE = (INSMOD_OPT_UNUSED << 8), /* C */
+	MODPROBE_OPT_BLACKLIST  = (INSMOD_OPT_UNUSED << 9) * ENABLE_FEATURE_MODPROBE_BLACKLIST,
 };
 
 struct globals {
@@ -116,6 +132,7 @@ static void add_probe(const char *name)
 		return;
 	}
 
+	DBG("queuing %s", name);
 	m->probed_name = name;
 	m->flags |= MODULE_FLAG_NEED_DEPS;
 	llist_add_to_end(&G.probes, m);
@@ -134,7 +151,7 @@ static int FAST_FUNC config_file_action(const char *filename,
 {
 	char *tokens[3];
 	parser_t *p;
-        struct module_entry *m;
+	struct module_entry *m;
 	int rc = TRUE;
 
 	if (bb_basename(filename)[0] == '.')
@@ -193,7 +210,7 @@ static int FAST_FUNC config_file_action(const char *filename,
 		}
 	}
 	config_close(p);
-error:
+ error:
 	return rc;
 }
 
@@ -203,14 +220,28 @@ static int read_config(const char *path)
 				config_file_action, NULL, NULL, 1);
 }
 
+static const char *humanly_readable_name(struct module_entry *m)
+{
+	/* probed_name may be NULL. modname always exists. */
+	return m->probed_name ? m->probed_name : m->modname;
+}
+
+/* Return: similar to bb_init_module:
+ * 0 on success,
+ * -errno on open/read error,
+ * errno on init_module() error
+ */
 static int do_modprobe(struct module_entry *m)
 {
-	struct module_entry *m2;
+	struct module_entry *m2 = m2; /* for compiler */
 	char *fn, *options;
-	int rc = -1;
+	int rc, first;
+	llist_t *l;
 
 	if (!(m->flags & MODULE_FLAG_FOUND_IN_MODDEP)) {
-		DBG("skipping %s, not found in modules.dep", m->modname);
+		if (!(option_mask32 & INSMOD_OPT_SILENT))
+			bb_error_msg("module %s not found in modules.dep",
+				humanly_readable_name(m));
 		return -ENOENT;
 	}
 	DBG("do_modprob'ing %s", m->modname);
@@ -218,37 +249,57 @@ static int do_modprobe(struct module_entry *m)
 	if (!(option_mask32 & MODPROBE_OPT_REMOVE))
 		m->deps = llist_rev(m->deps);
 
+	for (l = m->deps; l != NULL; l = l->link)
+		DBG("dep: %s", l->data);
+
+	first = 1;
 	rc = 0;
-	while (m->deps && rc == 0) {
-		fn = llist_pop(&m->deps);
+	while (m->deps) {
+		rc = 0;
+		fn = llist_pop(&m->deps); /* we leak it */
 		m2 = get_or_add_modentry(fn);
+
 		if (option_mask32 & MODPROBE_OPT_REMOVE) {
-			if (bb_delete_module(m->modname, O_EXCL) != 0)
-				rc = errno;
-		} else if (!(m2->flags & MODULE_FLAG_LOADED)) {
-			options = m2->options;
-			m2->options = NULL;
-			if (m == m2)
-				options = gather_options_str(options, G.cmdline_mopts);
-			rc = bb_init_module(fn, options);
-			DBG("loaded %s '%s', rc:%d", fn, options, rc);
-			if (rc == 0)
-				m2->flags |= MODULE_FLAG_LOADED;
-			free(options);
-		} else {
-			DBG("%s is already loaded, skipping", fn);
+			/* modprobe -r */
+			if (m2->flags & MODULE_FLAG_LOADED) {
+				rc = bb_delete_module(m2->modname, O_EXCL);
+				if (rc) {
+					if (first) {
+						bb_error_msg("failed to unload module %s: %s",
+							humanly_readable_name(m2),
+							moderror(rc));
+						break;
+					}
+				} else {
+					m2->flags &= ~MODULE_FLAG_LOADED;
+				}
+			}
+			/* do not error out if *deps* fail to unload */
+			first = 0;
+			continue;
 		}
 
-		free(fn);
-	}
+		if (m2->flags & MODULE_FLAG_LOADED) {
+			DBG("%s is already loaded, skipping", fn);
+			continue;
+		}
 
-//FIXME: what if rc < 0?
-	if (rc > 0 && !(option_mask32 & INSMOD_OPT_SILENT)) {
-		bb_error_msg("failed to %sload module %s: %s",
-			(option_mask32 & MODPROBE_OPT_REMOVE) ? "un" : "",
-			m->probed_name ? m->probed_name : m->modname,
-			moderror(rc)
-		);
+		options = m2->options;
+		m2->options = NULL;
+		if (m == m2)
+			options = gather_options_str(options, G.cmdline_mopts);
+		rc = bb_init_module(fn, options);
+		DBG("loaded %s '%s', rc:%d", fn, options, rc);
+		free(options);
+		if (rc) {
+			bb_error_msg("failed to load module %s (%s): %s",
+				humanly_readable_name(m2),
+				fn,
+				moderror(rc)
+			);
+			break;
+		}
+		m2->flags |= MODULE_FLAG_LOADED;
 	}
 
 	return rc;
@@ -260,7 +311,7 @@ static void load_modules_dep(void)
 	char *colon, *tokens[2];
 	parser_t *p;
 
-	/* Modprobe does not work at all without modprobe.dep,
+	/* Modprobe does not work at all without modules.dep,
 	 * even if the full module name is given. Returning error here
 	 * was making us later confuse user with this message:
 	 * "module /full/path/to/existing/file/module.ko not found".
@@ -293,8 +344,9 @@ static void load_modules_dep(void)
 			G.num_unresolved_deps--;
 			llist_add_to(&m->deps, xstrdup(tokens[0]));
 			if (tokens[1])
-				string_to_llist(tokens[1], &m->deps, " ");
-		}
+				string_to_llist(tokens[1], &m->deps, " \t");
+		} else
+			DBG("skipping dep line");
 	}
 	config_close(p);
 }
@@ -344,10 +396,12 @@ int modprobe_main(int argc UNUSED_PARAM, char **argv)
 	if (opt & (MODPROBE_OPT_INSERT_ALL | MODPROBE_OPT_REMOVE)) {
 		/* Each argument is a module name */
 		do {
+			DBG("adding module %s", *argv);
 			add_probe(*argv++);
 		} while (*argv);
 	} else {
 		/* First argument is module name, rest are parameters */
+		DBG("probing just module %s", *argv);
 		add_probe(argv[0]);
 		G.cmdline_mopts = parse_cmdline_module_options(argv);
 	}
@@ -366,35 +420,40 @@ int modprobe_main(int argc UNUSED_PARAM, char **argv)
 		load_modules_dep();
 	}
 
+	rc = 0;
 	while ((me = llist_pop(&G.probes)) != NULL) {
 		if (me->realnames == NULL) {
+			DBG("probing by module name");
 			/* This is not an alias. Literal names are blacklisted
 			 * only if '-b' is given.
 			 */
 			if (!(opt & MODPROBE_OPT_BLACKLIST)
 			 || !(me->flags & MODULE_FLAG_BLACKLISTED)
 			) {
-				rc = do_modprobe(me);
-//FIXME: what if rc > 0?
-				if (rc < 0 && !(opt & INSMOD_OPT_SILENT))
-					bb_error_msg("module %s not found",
-						     me->probed_name);
+				rc |= do_modprobe(me);
 			}
-		} else {
-			/* Probe all realnames */
-			do {
-				char *realname = llist_pop(&me->realnames);
-				struct module_entry *m2;
-
-				DBG("probing %s by realname %s", me->modname, realname);
-				m2 = get_or_add_modentry(realname);
-				if (!(m2->flags & MODULE_FLAG_BLACKLISTED))
-					do_modprobe(m2);
-//FIXME: error check?
-				free(realname);
-			} while (me->realnames != NULL);
+			continue;
 		}
+
+		/* Probe all real names for the alias */
+		do {
+			char *realname = llist_pop(&me->realnames);
+			struct module_entry *m2;
+
+			DBG("probing alias %s by realname %s", me->modname, realname);
+			m2 = get_or_add_modentry(realname);
+			if (!(m2->flags & MODULE_FLAG_BLACKLISTED)
+			 && (!(m2->flags & MODULE_FLAG_LOADED)
+			    || (opt & MODPROBE_OPT_REMOVE))
+			) {
+//TODO: we can pass "me" as 2nd param to do_modprobe,
+//and make do_modprobe emit more meaningful error messages
+//with alias name included, not just module name alias resolves to.
+				rc |= do_modprobe(m2);
+			}
+			free(realname);
+		} while (me->realnames != NULL);
 	}
 
-	return EXIT_SUCCESS;
+	return (rc != 0);
 }
