@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4: */
-#ifndef __LIBNETLINK_H__
-#define __LIBNETLINK_H__ 1
+#ifndef LIBNETLINK_H
+#define LIBNETLINK_H 1
 
 #include <linux/types.h>
 /* We need linux/types.h because older kernels use __u32 etc
@@ -8,13 +8,9 @@
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
 
+PUSH_AND_SET_FUNCTION_VISIBILITY_TO_HIDDEN
 
-#if __GNUC_PREREQ(4,1)
-# pragma GCC visibility push(hidden)
-#endif
-
-struct rtnl_handle
-{
+struct rtnl_handle {
 	int			fd;
 	struct sockaddr_nl	local;
 	struct sockaddr_nl	peer;
@@ -27,16 +23,16 @@ extern void rtnl_close(struct rtnl_handle *rth) FAST_FUNC;
 extern int xrtnl_wilddump_request(struct rtnl_handle *rth, int fam, int type) FAST_FUNC;
 extern int rtnl_dump_request(struct rtnl_handle *rth, int type, void *req, int len) FAST_FUNC;
 extern int xrtnl_dump_filter(struct rtnl_handle *rth,
-			int (*filter)(const struct sockaddr_nl*, struct nlmsghdr *n, void*),
-			void *arg1) FAST_FUNC;
+		int (*filter)(const struct sockaddr_nl*, struct nlmsghdr *n, void*) FAST_FUNC,
+		void *arg1) FAST_FUNC;
 
 /* bbox doesn't use parameters no. 3, 4, 6, 7, stub them out */
 #define rtnl_talk(rtnl, n, peer, groups, answer, junk, jarg) \
 	rtnl_talk(rtnl, n, answer)
 extern int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
-			unsigned groups, struct nlmsghdr *answer,
-			int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
-			void *jarg) FAST_FUNC;
+		unsigned groups, struct nlmsghdr *answer,
+		int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
+		void *jarg) FAST_FUNC;
 
 extern int rtnl_send(struct rtnl_handle *rth, char *buf, int) FAST_FUNC;
 
@@ -48,8 +44,6 @@ extern int rta_addattr_l(struct rtattr *rta, int maxlen, int type, void *data, i
 
 extern int parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta, int len) FAST_FUNC;
 
-#if __GNUC_PREREQ(4,1)
-# pragma GCC visibility pop
-#endif
+POP_SAVED_FUNCTION_VISIBILITY
 
-#endif /* __LIBNETLINK_H__ */
+#endif
