@@ -39,9 +39,9 @@ static struct idxmap *find_by_index(int idx)
 	return NULL;
 }
 
-int ll_remember_index(const struct sockaddr_nl *who ATTRIBUTE_UNUSED,
+int FAST_FUNC ll_remember_index(const struct sockaddr_nl *who UNUSED_PARAM,
 		struct nlmsghdr *n,
-		void *arg ATTRIBUTE_UNUSED)
+		void *arg UNUSED_PARAM)
 {
 	int h;
 	struct ifinfomsg *ifi = NLMSG_DATA(n);
@@ -86,7 +86,7 @@ int ll_remember_index(const struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 	return 0;
 }
 
-const char *ll_idx_n2a(int idx, char *buf)
+const char FAST_FUNC *ll_idx_n2a(int idx, char *buf)
 {
 	struct idxmap *im;
 
@@ -100,7 +100,7 @@ const char *ll_idx_n2a(int idx, char *buf)
 }
 
 
-const char *ll_index_to_name(int idx)
+const char FAST_FUNC *ll_index_to_name(int idx)
 {
 	static char nbuf[16];
 
@@ -121,7 +121,7 @@ int ll_index_to_type(int idx)
 }
 #endif
 
-unsigned ll_index_to_flags(int idx)
+unsigned FAST_FUNC ll_index_to_flags(int idx)
 {
 	struct idxmap *im;
 
@@ -133,7 +133,7 @@ unsigned ll_index_to_flags(int idx)
 	return 0;
 }
 
-int xll_name_to_index(const char *const name)
+int FAST_FUNC xll_name_to_index(const char *name)
 {
 	int ret = 0;
 	int sock_fd;
@@ -172,11 +172,11 @@ int xll_name_to_index(const char *const name)
 #endif
 
 	sock_fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (sock_fd) {
+	if (sock_fd >= 0) {
 		struct ifreq ifr;
 		int tmp;
 
-		strncpy(ifr.ifr_name, name, IFNAMSIZ);
+		strncpy_IFNAMSIZ(ifr.ifr_name, name);
 		ifr.ifr_ifindex = -1;
 		tmp = ioctl(sock_fd, SIOCGIFINDEX, &ifr);
 		close(sock_fd);
@@ -192,7 +192,7 @@ int xll_name_to_index(const char *const name)
 	return ret;
 }
 
-int ll_init_map(struct rtnl_handle *rth)
+int FAST_FUNC ll_init_map(struct rtnl_handle *rth)
 {
 	xrtnl_wilddump_request(rth, AF_UNSPEC, RTM_GETLINK);
 	xrtnl_dump_filter(rth, ll_remember_index, &idxmap);
