@@ -7,9 +7,7 @@
  * Licensed under GPLv2, see file LICENSE in this tarball for details.
  */
 
-#if __GNUC_PREREQ(4,1)
-# pragma GCC visibility push(hidden)
-#endif
+PUSH_AND_SET_FUNCTION_VISIBILITY_TO_HIDDEN
 
 struct hcc {
 	char *ip;
@@ -32,6 +30,4 @@ void ipsvd_perhost_remove(int pid);
 //unsigned ipsvd_perhost_setpid(int pid);
 //void ipsvd_perhost_free(void);
 
-#if __GNUC_PREREQ(4,1)
-# pragma GCC visibility pop
-#endif
+POP_SAVED_FUNCTION_VISIBILITY
