@@ -11,14 +11,14 @@
  *
  * The telnetd manpage says it all:
  *
- *   Telnetd operates by allocating a pseudo-terminal device (see pty(4))  for
- *   a client, then creating a login process which has the slave side of the
- *   pseudo-terminal as stdin, stdout, and stderr. Telnetd manipulates the
- *   master side of the pseudo-terminal, implementing the telnet protocol and
- *   passing characters between the remote client and the login process.
+ * Telnetd operates by allocating a pseudo-terminal device (see pty(4)) for
+ * a client, then creating a login process which has the slave side of the
+ * pseudo-terminal as stdin, stdout, and stderr. Telnetd manipulates the
+ * master side of the pseudo-terminal, implementing the telnet protocol and
+ * passing characters between the remote client and the login process.
  *
  * Vladimir Oleynik <dzo@simtreas.ru> 2001
- *     Set process group corrections, initial busybox port
+ * Set process group corrections, initial busybox port
  */
 
 #define DEBUG 0
@@ -32,18 +32,19 @@
 #endif
 #include <arpa/telnet.h>
 
-/* Structure that describes a session */
 struct tsession {
 	struct tsession *next;
-	int sockfd_read, sockfd_write, ptyfd;
-	int shell_pid;
+	pid_t shell_pid;
+	int sockfd_read;
+	int sockfd_write;
+	int ptyfd;
 
 	/* two circular buffers */
 	/*char *buf1, *buf2;*/
-/*#define TS_BUF1 ts->buf1*/
-/*#define TS_BUF2 TS_BUF2*/
-#define TS_BUF1 ((unsigned char*)(ts + 1))
-#define TS_BUF2 (((unsigned char*)(ts + 1)) + BUFSIZE)
+/*#define TS_BUF1(ts) ts->buf1*/
+/*#define TS_BUF2(ts) TS_BUF2(ts)*/
+#define TS_BUF1(ts) ((unsigned char*)(ts + 1))
+#define TS_BUF2(ts) (((unsigned char*)(ts + 1)) + BUFSIZE)
 	int rdidx1, wridx1, size1;
 	int rdidx2, wridx2, size2;
 };
@@ -54,10 +55,17 @@ enum { BUFSIZE = (4 * 1024 - sizeof(struct tsession)) / 2 };
 
 
 /* Globals */
-static int maxfd;
-static struct tsession *sessions;
-static const char *loginpath = "/bin/login";
-static const char *issuefile = "/etc/issue.net";
+struct globals {
+	struct tsession *sessions;
+	const char *loginpath;
+	const char *issuefile;
+	int maxfd;
+};
+#define G (*(struct globals*)&bb_common_bufsiz1)
+#define INIT_G() do { \
+	G.loginpath = "/bin/login"; \
+	G.issuefile = "/etc/issue.net"; \
+} while (0)
 
 
 /*
@@ -74,9 +82,6 @@ static const char *issuefile = "/etc/issue.net";
    past (bf + len) then that IAC will be left unprocessed and *processed
    will be less than len.
 
-   FIXME - if we mean to send 0xFF to the terminal then it will be escaped,
-   what is the escape character?  We aren't handling that situation here.
-
    CR-LF ->'s CR mapping is also done here, for convenience.
 
    NB: may fail to remove iacs which wrap around buffer!
@@ -84,7 +89,7 @@ static const char *issuefile = "/etc/issue.net";
 static unsigned char *
 remove_iacs(struct tsession *ts, int *pnum_totty)
 {
-	unsigned char *ptr0 = TS_BUF1 + ts->wridx1;
+	unsigned char *ptr0 = TS_BUF1(ts) + ts->wridx1;
 	unsigned char *ptr = ptr0;
 	unsigned char *totty = ptr;
 	unsigned char *end = ptr + MIN(BUFSIZE - ts->wridx1, ts->size1);
@@ -121,9 +126,9 @@ remove_iacs(struct tsession *ts, int *pnum_totty)
 		 * TELOPT_NAWS support!
 		 */
 		if ((ptr+2) >= end) {
-			/* only the beginning of the IAC is in the
+			/* Only the beginning of the IAC is in the
 			buffer we were asked to process, we can't
-			process this char. */
+			process this char */
 			break;
 		}
 		/*
@@ -149,21 +154,71 @@ remove_iacs(struct tsession *ts, int *pnum_totty)
 
 	num_totty = totty - ptr0;
 	*pnum_totty = num_totty;
-	/* the difference between ptr and totty is number of iacs
-	   we removed from the stream. Adjust buf1 accordingly. */
+	/* The difference between ptr and totty is number of iacs
+	   we removed from the stream. Adjust buf1 accordingly */
 	if ((ptr - totty) == 0) /* 99.999% of cases */
 		return ptr0;
 	ts->wridx1 += ptr - totty;
 	ts->size1 -= ptr - totty;
-	/* move chars meant for the terminal towards the end of the buffer */
+	/* Move chars meant for the terminal towards the end of the buffer */
 	return memmove(ptr - num_totty, ptr0, num_totty);
 }
 
+/*
+ * Converting single IAC into double on output
+ */
+static size_t iac_safe_write(int fd, const char *buf, size_t count)
+{
+	const char *IACptr;
+	size_t wr, rc, total;
+
+	total = 0;
+	while (1) {
+		if (count == 0)
+			return total;
+		if (*buf == (char)IAC) {
+			static const char IACIAC[] ALIGN1 = { IAC, IAC };
+			rc = safe_write(fd, IACIAC, 2);
+			if (rc != 2)
+				break;
+			buf++;
+			total++;
+			count--;
+			continue;
+		}
+		/* count != 0, *buf != IAC */
+		IACptr = memchr(buf, IAC, count);
+		wr = count;
+		if (IACptr)
+			wr = IACptr - buf;
+		rc = safe_write(fd, buf, wr);
+		if (rc != wr)
+			break;
+		buf += rc;
+		total += rc;
+		count -= rc;
+	}
+	/* here: rc - result of last short write */
+	if ((ssize_t)rc < 0) { /* error? */
+		if (total == 0)
+			return rc;
+		rc = 0;
+	}
+	return total + rc;
+}
+
+/* Must match getopt32 string */
+enum {
+	OPT_WATCHCHILD = (1 << 2), /* -K */
+	OPT_INETD      = (1 << 3) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -i */
+	OPT_PORT       = (1 << 4) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -p */
+	OPT_FOREGROUND = (1 << 6) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -F */
+};
 
 static struct tsession *
 make_new_session(
-		USE_FEATURE_TELNETD_STANDALONE(int sock)
-		SKIP_FEATURE_TELNETD_STANDALONE(void)
+		IF_FEATURE_TELNETD_STANDALONE(int sock)
+		IF_NOT_FEATURE_TELNETD_STANDALONE(void)
 ) {
 	const char *login_argv[2];
 	struct termios termbuf;
@@ -174,24 +229,26 @@ make_new_session(
 	/*ts->buf1 = (char *)(ts + 1);*/
 	/*ts->buf2 = ts->buf1 + BUFSIZE;*/
 
-	/* Got a new connection, set up a tty. */
+	/* Got a new connection, set up a tty */
 	fd = xgetpty(tty_name);
-	if (fd > maxfd)
-		maxfd = fd;
+	if (fd > G.maxfd)
+		G.maxfd = fd;
 	ts->ptyfd = fd;
 	ndelay_on(fd);
+	close_on_exec_on(fd);
+
 #if ENABLE_FEATURE_TELNETD_STANDALONE
-	ts->sockfd_read = sock;
 	/* SO_KEEPALIVE by popular demand */
 	setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &const_int_1, sizeof(const_int_1));
+	ts->sockfd_read = sock;
 	ndelay_on(sock);
-	if (!sock) { /* We are called with fd 0 - we are in inetd mode */
+	if (sock == 0) { /* We are called with fd 0 - we are in inetd mode */
 		sock++; /* so use fd 1 for output */
 		ndelay_on(sock);
 	}
 	ts->sockfd_write = sock;
-	if (sock > maxfd)
-		maxfd = sock;
+	if (sock > G.maxfd)
+		G.maxfd = sock;
 #else
 	/* SO_KEEPALIVE by popular demand */
 	setsockopt(0, SOL_SOCKET, SO_KEEPALIVE, &const_int_1, sizeof(const_int_1));
@@ -200,6 +257,7 @@ make_new_session(
 	ndelay_on(0);
 	ndelay_on(1);
 #endif
+
 	/* Make the telnet client understand we will echo characters so it
 	 * should not do it locally. We don't tell the client to run linemode,
 	 * because we want to handle line editing and tab completion and other
@@ -208,13 +266,24 @@ make_new_session(
 		static const char iacs_to_send[] ALIGN1 = {
 			IAC, DO, TELOPT_ECHO,
 			IAC, DO, TELOPT_NAWS,
-			IAC, DO, TELOPT_LFLOW,
+		/* This requires telnetd.ctrlSQ.patch (incomplete) */
+		/*	IAC, DO, TELOPT_LFLOW, */
 			IAC, WILL, TELOPT_ECHO,
 			IAC, WILL, TELOPT_SGA
 		};
-		memcpy(TS_BUF2, iacs_to_send, sizeof(iacs_to_send));
-		ts->rdidx2 = sizeof(iacs_to_send);
-		ts->size2 = sizeof(iacs_to_send);
+		/* This confuses iac_safe_write(), it will try to duplicate
+		 * each IAC... */
+		//memcpy(TS_BUF2(ts), iacs_to_send, sizeof(iacs_to_send));
+		//ts->rdidx2 = sizeof(iacs_to_send);
+		//ts->size2 = sizeof(iacs_to_send);
+		/* So just stuff it into TCP stream! (no error check...) */
+#if ENABLE_FEATURE_TELNETD_STANDALONE
+		safe_write(sock, iacs_to_send, sizeof(iacs_to_send));
+#else
+		safe_write(1, iacs_to_send, sizeof(iacs_to_send));
+#endif
+		/*ts->rdidx2 = 0; - xzalloc did it */
+		/*ts->size2 = 0;*/
 	}
 
 	fflush(NULL); /* flush all streams */
@@ -235,13 +304,13 @@ make_new_session(
 	/* Child */
 	/* Careful - we are after vfork! */
 
-	/* make new session and process group */
-	setsid();
-
-	/* Restore default signal handling */
+	/* Restore default signal handling ASAP */
 	bb_signals((1 << SIGCHLD) + (1 << SIGPIPE), SIG_DFL);
 
-	/* open the child's side of the tty. */
+	/* Make new session and process group */
+	setsid();
+
+	/* Open the child's side of the tty */
 	/* NB: setsid() disconnects from any previous ctty's. Therefore
 	 * we must open child's side of the tty AFTER setsid! */
 	close(0);
@@ -250,8 +319,8 @@ make_new_session(
 	xdup2(0, 2);
 	tcsetpgrp(0, getpid()); /* switch this tty's process group to us */
 
-	/* The pseudo-terminal allocated to the client is configured to operate in
-	 * cooked mode, and with XTABS CRMOD enabled (see tty(4)). */
+	/* The pseudo-terminal allocated to the client is configured to operate
+	 * in cooked mode, and with XTABS CRMOD enabled (see tty(4)) */
 	tcgetattr(0, &termbuf);
 	termbuf.c_lflag |= ECHO; /* if we use readline we dont want this */
 	termbuf.c_oflag |= ONLCR | XTABS;
@@ -266,40 +335,35 @@ make_new_session(
 	 * issue files, and they may block writing to fd 1,
 	 * (parent is supposed to read it, but parent waits
 	 * for vforked child to exec!) */
-	print_login_issue(issuefile, tty_name);
+	print_login_issue(G.issuefile, tty_name);
 
 	/* Exec shell / login / whatever */
-	login_argv[0] = loginpath;
+	login_argv[0] = G.loginpath;
 	login_argv[1] = NULL;
 	/* exec busybox applet (if PREFER_APPLETS=y), if that fails,
-	 * exec external program */
-	BB_EXECVP(loginpath, (char **)login_argv);
+	 * exec external program.
+	 * NB: sock is either 0 or has CLOEXEC set on it.
+	 * fd has CLOEXEC set on it too. These two fds will be closed here.
+	 */
+	BB_EXECVP(G.loginpath, (char **)login_argv);
 	/* _exit is safer with vfork, and we shouldn't send message
 	 * to remote clients anyway */
-	_exit(EXIT_FAILURE); /*bb_perror_msg_and_die("execv %s", loginpath);*/
+	_exit(EXIT_FAILURE); /*bb_perror_msg_and_die("execv %s", G.loginpath);*/
 }
 
-/* Must match getopt32 string */
-enum {
-	OPT_WATCHCHILD = (1 << 2), /* -K */
-	OPT_INETD      = (1 << 3) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -i */
-	OPT_PORT       = (1 << 4) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -p */
-	OPT_FOREGROUND = (1 << 6) * ENABLE_FEATURE_TELNETD_STANDALONE, /* -F */
-};
-
 #if ENABLE_FEATURE_TELNETD_STANDALONE
 
 static void
 free_session(struct tsession *ts)
 {
-	struct tsession *t = sessions;
+	struct tsession *t = G.sessions;
 
 	if (option_mask32 & OPT_INETD)
 		exit(EXIT_SUCCESS);
 
 	/* Unlink this telnet session from the session list */
 	if (t == ts)
-		sessions = ts->next;
+		G.sessions = ts->next;
 	else {
 		while (t->next != ts)
 			t = t->next;
@@ -311,7 +375,7 @@ free_session(struct tsession *ts)
 	 * doesn't send SIGKILL. When we close ptyfd,
 	 * kernel sends SIGHUP to processes having slave side opened. */
 	kill(ts->shell_pid, SIGKILL);
-	wait4(ts->shell_pid, NULL, 0, NULL);
+	waitpid(ts->shell_pid, NULL, 0);
 #endif
 	close(ts->ptyfd);
 	close(ts->sockfd_read);
@@ -321,17 +385,17 @@ free_session(struct tsession *ts)
 	free(ts);
 
 	/* Scan all sessions and find new maxfd */
-	maxfd = 0;
-	ts = sessions;
+	G.maxfd = 0;
+	ts = G.sessions;
 	while (ts) {
-		if (maxfd < ts->ptyfd)
-			maxfd = ts->ptyfd;
-		if (maxfd < ts->sockfd_read)
-			maxfd = ts->sockfd_read;
+		if (G.maxfd < ts->ptyfd)
+			G.maxfd = ts->ptyfd;
+		if (G.maxfd < ts->sockfd_read)
+			G.maxfd = ts->sockfd_read;
 #if 0
 		/* Again, sockfd_write == sockfd_read here */
-		if (maxfd < ts->sockfd_write)
-			maxfd = ts->sockfd_write;
+		if (G.maxfd < ts->sockfd_write)
+			G.maxfd = ts->sockfd_write;
 #endif
 		ts = ts->next;
 	}
@@ -354,7 +418,7 @@ static void handle_sigchld(int sig UNUSED_PARAM)
 		pid = wait_any_nohang(NULL);
 		if (pid <= 0)
 			break;
-		ts = sessions;
+		ts = G.sessions;
 		while (ts) {
 			if (ts->shell_pid == pid) {
 				ts->shell_pid = -1;
@@ -385,11 +449,13 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		portnbr = 23,
 	};
 #endif
+	INIT_G();
+
 	/* Even if !STANDALONE, we accept (and ignore) -i, thus people
 	 * don't need to guess whether it's ok to pass -i to us */
-	opt = getopt32(argv, "f:l:Ki" USE_FEATURE_TELNETD_STANDALONE("p:b:F"),
-			&issuefile, &loginpath
-			USE_FEATURE_TELNETD_STANDALONE(, &opt_portnbr, &opt_bindaddr));
+	opt = getopt32(argv, "f:l:Ki" IF_FEATURE_TELNETD_STANDALONE("p:b:F"),
+			&G.issuefile, &G.loginpath
+			IF_FEATURE_TELNETD_STANDALONE(, &opt_portnbr, &opt_bindaddr));
 	if (!IS_INETD /*&& !re_execed*/) {
 		/* inform that we start in standalone mode?
 		 * May be useful when people forget to give -i */
@@ -402,29 +468,30 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 	}
 	/* Redirect log to syslog early, if needed */
 	if (IS_INETD || !(opt & OPT_FOREGROUND)) {
-		openlog(applet_name, 0, LOG_USER);
+		openlog(applet_name, LOG_PID, LOG_DAEMON);
 		logmode = LOGMODE_SYSLOG;
 	}
-	USE_FEATURE_TELNETD_STANDALONE(
+	IF_FEATURE_TELNETD_STANDALONE(
 		if (opt & OPT_PORT)
 			portnbr = xatou16(opt_portnbr);
 	);
 
-	/* Used to check access(loginpath, X_OK) here. Pointless.
+	/* Used to check access(G.loginpath, X_OK) here. Pointless.
 	 * exec will do this for us for free later. */
 
 #if ENABLE_FEATURE_TELNETD_STANDALONE
 	if (IS_INETD) {
-		sessions = make_new_session(0);
-		if (!sessions) /* pty opening or vfork problem, exit */
+		G.sessions = make_new_session(0);
+		if (!G.sessions) /* pty opening or vfork problem, exit */
 			return 1; /* make_new_session prints error message */
 	} else {
 		master_fd = create_and_bind_stream_or_die(opt_bindaddr, portnbr);
 		xlisten(master_fd, 1);
+		close_on_exec_on(master_fd);
 	}
 #else
-	sessions = make_new_session();
-	if (!sessions) /* pty opening or vfork problem, exit */
+	G.sessions = make_new_session();
+	if (!G.sessions) /* pty opening or vfork problem, exit */
 		return 1; /* make_new_session prints error message */
 #endif
 
@@ -437,8 +504,8 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		signal(SIGCHLD, SIG_IGN);
 
 /*
-   This is how the buffers are used. The arrows indicate the movement
-   of data.
+   This is how the buffers are used. The arrows indicate data flow.
+
    +-------+     wridx1++     +------+     rdidx1++     +----------+
    |       | <--------------  | buf1 | <--------------  |          |
    |       |     size1--      +------+     size1++      |          |
@@ -462,9 +529,9 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 	 * ptys if there is room in their session buffers.
 	 * NB: scalability problem: we recalculate entire bitmap
 	 * before each select. Can be a problem with 500+ connections. */
-	ts = sessions;
+	ts = G.sessions;
 	while (ts) {
-		struct tsession *next = ts->next; /* in case we free ts. */
+		struct tsession *next = ts->next; /* in case we free ts */
 		if (ts->shell_pid == -1) {
 			/* Child died and we detected that */
 			free_session(ts);
@@ -485,16 +552,16 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		/* This is needed because free_session() does not
 		 * take master_fd into account when it finds new
 		 * maxfd among remaining fd's */
-		if (master_fd > maxfd)
-			maxfd = master_fd;
+		if (master_fd > G.maxfd)
+			G.maxfd = master_fd;
 	}
 
-	count = select(maxfd + 1, &rdfdset, &wrfdset, NULL, NULL);
+	count = select(G.maxfd + 1, &rdfdset, &wrfdset, NULL, NULL);
 	if (count < 0)
 		goto again; /* EINTR or ENOMEM */
 
 #if ENABLE_FEATURE_TELNETD_STANDALONE
-	/* First check for and accept new sessions. */
+	/* Check for and accept new sessions */
 	if (!IS_INETD && FD_ISSET(master_fd, &rdfdset)) {
 		int fd;
 		struct tsession *new_ts;
@@ -502,26 +569,28 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		fd = accept(master_fd, NULL, NULL);
 		if (fd < 0)
 			goto again;
-		/* Create a new session and link it into our active list */
+		close_on_exec_on(fd);
+
+		/* Create a new session and link it into active list */
 		new_ts = make_new_session(fd);
 		if (new_ts) {
-			new_ts->next = sessions;
-			sessions = new_ts;
+			new_ts->next = G.sessions;
+			G.sessions = new_ts;
 		} else {
 			close(fd);
 		}
 	}
 #endif
 
-	/* Then check for data tunneling. */
-	ts = sessions;
+	/* Then check for data tunneling */
+	ts = G.sessions;
 	while (ts) { /* For all sessions... */
-		struct tsession *next = ts->next; /* in case we free ts. */
+		struct tsession *next = ts->next; /* in case we free ts */
 
 		if (/*ts->size1 &&*/ FD_ISSET(ts->ptyfd, &wrfdset)) {
 			int num_totty;
 			unsigned char *ptr;
-			/* Write to pty from buffer 1. */
+			/* Write to pty from buffer 1 */
 			ptr = remove_iacs(ts, &num_totty);
 			count = safe_write(ts->ptyfd, ptr, num_totty);
 			if (count < 0) {
@@ -536,9 +605,9 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		}
  skip1:
 		if (/*ts->size2 &&*/ FD_ISSET(ts->sockfd_write, &wrfdset)) {
-			/* Write to socket from buffer 2. */
+			/* Write to socket from buffer 2 */
 			count = MIN(BUFSIZE - ts->wridx2, ts->size2);
-			count = safe_write(ts->sockfd_write, TS_BUF2 + ts->wridx2, count);
+			count = iac_safe_write(ts->sockfd_write, (void*)(TS_BUF2(ts) + ts->wridx2), count);
 			if (count < 0) {
 				if (errno == EAGAIN)
 					goto skip2;
@@ -566,16 +635,16 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		}
 
 		if (/*ts->size1 < BUFSIZE &&*/ FD_ISSET(ts->sockfd_read, &rdfdset)) {
-			/* Read from socket to buffer 1. */
+			/* Read from socket to buffer 1 */
 			count = MIN(BUFSIZE - ts->rdidx1, BUFSIZE - ts->size1);
-			count = safe_read(ts->sockfd_read, TS_BUF1 + ts->rdidx1, count);
+			count = safe_read(ts->sockfd_read, TS_BUF1(ts) + ts->rdidx1, count);
 			if (count <= 0) {
 				if (count < 0 && errno == EAGAIN)
 					goto skip3;
 				goto kill_session;
 			}
 			/* Ignore trailing NUL if it is there */
-			if (!TS_BUF1[ts->rdidx1 + count - 1]) {
+			if (!TS_BUF1(ts)[ts->rdidx1 + count - 1]) {
 				--count;
 			}
 			ts->size1 += count;
@@ -585,9 +654,9 @@ int telnetd_main(int argc UNUSED_PARAM, char **argv)
 		}
  skip3:
 		if (/*ts->size2 < BUFSIZE &&*/ FD_ISSET(ts->ptyfd, &rdfdset)) {
-			/* Read from pty to buffer 2. */
+			/* Read from pty to buffer 2 */
 			count = MIN(BUFSIZE - ts->rdidx2, BUFSIZE - ts->size2);
-			count = safe_read(ts->ptyfd, TS_BUF2 + ts->rdidx2, count);
+			count = safe_read(ts->ptyfd, TS_BUF2(ts) + ts->rdidx2, count);
 			if (count <= 0) {
 				if (count < 0 && errno == EAGAIN)
 					goto skip4;
