@@ -404,6 +404,8 @@ int conf_write(const char *name)
 			if (ret == 0)
 				strftime(buf, sizeof(buf), "#define AUTOCONF_TIMESTAMP "
 					"\"%Y-%m-%d %H:%M:%S\"\n", localtime(&now));
+		} else { /* bbox */
+			strcpy(buf, "#define AUTOCONF_TIMESTAMP \"\"\n");
 		}
 		fprintf(out_h, "/*\n"
 			       " * Automatically generated C config: don't edit\n"
@@ -435,7 +437,7 @@ int conf_write(const char *name)
 					       " */\n", str);
 		} else if (!(sym->flags & SYMBOL_CHOICE)) {
 			sym_calc_value(sym);
-/* bbox: we want to all syms
+/* bbox: we want to see all syms
 			if (!(sym->flags & SYMBOL_WRITE))
 				goto next;
 */
@@ -456,8 +458,8 @@ int conf_write(const char *name)
 						fprintf(out_h, "#undef CONFIG_%s\n", sym->name);
 						/* bbox */
 						fprintf(out_h, "#define ENABLE_%s 0\n", sym->name);
-						fprintf(out_h, "#define USE_%s(...)\n", sym->name);
-						fprintf(out_h, "#define SKIP_%s(...) __VA_ARGS__\n", sym->name);
+						fprintf(out_h, "#define IF_%s(...)\n", sym->name);
+						fprintf(out_h, "#define IF_NOT_%s(...) __VA_ARGS__\n", sym->name);
 					}
 					break;
 				case mod:
@@ -471,8 +473,8 @@ int conf_write(const char *name)
 						fprintf(out_h, "#define CONFIG_%s 1\n", sym->name);
 						/* bbox */
 						fprintf(out_h, "#define ENABLE_%s 1\n", sym->name);
-						fprintf(out_h, "#define USE_%s(...) __VA_ARGS__\n", sym->name);
-						fprintf(out_h, "#define SKIP_%s(...)\n", sym->name);
+						fprintf(out_h, "#define IF_%s(...) __VA_ARGS__\n", sym->name);
+						fprintf(out_h, "#define IF_NOT_%s(...)\n", sym->name);
 					}
 					break;
 				}
@@ -503,8 +505,8 @@ int conf_write(const char *name)
 					fputs("\"\n", out_h);
 					/* bbox */
 					fprintf(out_h, "#define ENABLE_%s 1\n", sym->name);
-					fprintf(out_h, "#define USE_%s(...) __VA_ARGS__\n", sym->name);
-					fprintf(out_h, "#define SKIP_%s(...)\n", sym->name);
+					fprintf(out_h, "#define IF_%s(...) __VA_ARGS__\n", sym->name);
+					fprintf(out_h, "#define IF_NOT_%s(...)\n", sym->name);
 				}
 				break;
 			case S_HEX:
@@ -515,20 +517,22 @@ int conf_write(const char *name)
 						fprintf(out_h, "#define CONFIG_%s 0x%s\n", sym->name, str);
 						/* bbox */
 						fprintf(out_h, "#define ENABLE_%s 1\n", sym->name);
-						fprintf(out_h, "#define USE_%s(...) __VA_ARGS__\n", sym->name);
-						fprintf(out_h, "#define SKIP_%s(...)\n", sym->name);
+						fprintf(out_h, "#define IF_%s(...) __VA_ARGS__\n", sym->name);
+						fprintf(out_h, "#define IF_NOT_%s(...)\n", sym->name);
 					}
 					break;
 				}
 			case S_INT:
 				str = sym_get_string_value(sym);
+				if (!str[0])
+					str = "0";
 				fprintf(out, "CONFIG_%s=%s\n", sym->name, str);
 				if (out_h) {
 					fprintf(out_h, "#define CONFIG_%s %s\n", sym->name, str);
 					/* bbox */
 					fprintf(out_h, "#define ENABLE_%s 1\n", sym->name);
-					fprintf(out_h, "#define USE_%s(...) __VA_ARGS__\n", sym->name);
-					fprintf(out_h, "#define SKIP_%s(...)\n", sym->name);
+					fprintf(out_h, "#define IF_%s(...) __VA_ARGS__\n", sym->name);
+					fprintf(out_h, "#define IF_NOT_%s(...)\n", sym->name);
 				}
 				break;
 			}
