@@ -25,6 +25,8 @@
  *
  * Port to busybox: KaiGai Kohei <kaigai@kaigai.gr.jp>
  *                  - based on coreutils-5.97 (in Fedora Core 6)
+ *
+ * Licensed under GPLv2, see file LICENSE in this tarball for details.
  */
 #include <getopt.h>
 #include <selinux/context.h>
@@ -127,10 +129,10 @@ int runcon_main(int argc UNUSED_PARAM, char **argv)
 				     context_str(con));
 
 	if (setexeccon(context_str(con)))
-		bb_error_msg_and_die("cannot set up security context '%s'",
+		bb_error_msg_and_die("can't set up security context '%s'",
 				     context_str(con));
 
 	execvp(argv[0], argv);
 
-	bb_perror_msg_and_die("cannot execute '%s'", argv[0]);
+	bb_perror_msg_and_die("can't execute '%s'", argv[0]);
 }
