@@ -1,4 +1,3 @@
-#if ENABLE_FEATURE_OSF_LABEL
 /*
  * Copyright (c) 1987, 1988 Regents of the University of California.
  * All rights reserved.
@@ -32,6 +31,7 @@
  * SUCH DAMAGE.
  */
 
+#if ENABLE_FEATURE_OSF_LABEL
 
 #ifndef BSD_DISKMAGIC
 #define BSD_DISKMAGIC     ((uint32_t) 0x82564557)
@@ -45,18 +45,16 @@
 
 #if defined(i386) || defined(__sparc__) || defined(__arm__) \
  || defined(__m68k__) || defined(__mips__) || defined(__s390__) \
- || defined(__sh__) || defined(__x86_64__)
-#define BSD_LABELSECTOR   1
-#define BSD_LABELOFFSET   0
+ || defined(__s390__) || defined(__s390x__) \
+ || defined(__sh__) || defined(__x86_64__) || defined(__avr32__)
+# define BSD_LABELSECTOR   1
+# define BSD_LABELOFFSET   0
 #elif defined(__alpha__) || defined(__powerpc__) || defined(__ia64__) \
  || defined(__hppa__)
-#define BSD_LABELSECTOR   0
-#define BSD_LABELOFFSET   64
-#elif defined(__s390__) || defined(__s390x__)
-#define BSD_LABELSECTOR   1
-#define BSD_LABELOFFSET   0
+# define BSD_LABELSECTOR   0
+# define BSD_LABELOFFSET   64
 #else
-#error unknown architecture
+# error unknown architecture
 #endif
 
 #define BSD_BBSIZE        8192          /* size of boot area, with label */
@@ -688,13 +686,12 @@ xbsd_get_bootstrap(char *path, void *ptr, int size)
 {
 	int fdb;
 
-	fdb = open(path, O_RDONLY);
+	fdb = open_or_warn(path, O_RDONLY);
 	if (fdb < 0) {
-		perror(path);
 		return 0;
 	}
-	if (read(fdb, ptr, size) < 0) {
-		perror(path);
+	if (full_read(fdb, ptr, size) < 0) {
+		bb_simple_perror_msg(path);
 		close(fdb);
 		return 0;
 	}
@@ -765,10 +762,8 @@ xbsd_write_bootstrap(void)
 	sector = get_start_sect(xbsd_part);
 #endif
 
-	if (lseek(dev_fd, sector * SECTOR_SIZE, SEEK_SET) == -1)
-		fdisk_fatal(unable_to_seek);
-	if (BSD_BBSIZE != write(dev_fd, disklabelbuffer, BSD_BBSIZE))
-		fdisk_fatal(unable_to_write);
+	seek_sector(sector);
+	xwrite(dev_fd, disklabelbuffer, BSD_BBSIZE);
 
 #if defined(__alpha__)
 	printf("Bootstrap installed on %s\n", disk_device);
@@ -938,9 +933,8 @@ xbsd_readlabel(struct partition *p)
 	sector = 0;
 #endif
 
-	if (lseek(dev_fd, sector * SECTOR_SIZE, SEEK_SET) == -1)
-		fdisk_fatal(unable_to_seek);
-	if (BSD_BBSIZE != read(dev_fd, disklabelbuffer, BSD_BBSIZE))
+	seek_sector(sector);
+	if (BSD_BBSIZE != full_read(dev_fd, disklabelbuffer, BSD_BBSIZE))
 		fdisk_fatal(unable_to_read);
 
 	memmove(d, &disklabelbuffer[BSD_LABELSECTOR * SECTOR_SIZE + BSD_LABELOFFSET],
@@ -984,15 +978,12 @@ xbsd_writelabel(struct partition *p)
 
 #if defined(__alpha__) && BSD_LABELSECTOR == 0
 	alpha_bootblock_checksum(disklabelbuffer);
-	if (lseek(dev_fd, 0, SEEK_SET) == -1)
-		fdisk_fatal(unable_to_seek);
-	if (BSD_BBSIZE != write(dev_fd, disklabelbuffer, BSD_BBSIZE))
-		fdisk_fatal(unable_to_write);
+	seek_sector(0);
+	xwrite(dev_fd, disklabelbuffer, BSD_BBSIZE);
 #else
-	if (lseek(dev_fd, sector * SECTOR_SIZE + BSD_LABELOFFSET, SEEK_SET) == -1)
-		fdisk_fatal(unable_to_seek);
-	if (sizeof(struct xbsd_disklabel) != write(dev_fd, d, sizeof(struct xbsd_disklabel)))
-		fdisk_fatal(unable_to_write);
+	seek_sector(sector);
+	lseek(dev_fd, BSD_LABELOFFSET, SEEK_CUR);
+	xwrite(dev_fd, d, sizeof(*d));
 #endif
 	sync_disks();
 	return 1;
