@@ -3,6 +3,8 @@
  *
  * This version was taken from util-linux and scrubbed down for busybox.
  *
+ * Licensed under GPLv2, see file LICENSE in this tarball for details.
+ *
  * This uses cross-platform Linux interfaces to enter a system sleep state,
  * and leave it no later than a specified time.  It uses any RTC framework
  * driver that supports standard driver model wakeup flags.
@@ -112,7 +114,7 @@ int rtcwake_main(int argc UNUSED_PARAM, char **argv)
 	int utc = -1;
 	int fd;
 
-#if ENABLE_GETOPT_LONG
+#if ENABLE_LONG_OPTS
 	static const char rtcwake_longopts[] ALIGN1 =
 		"auto\0"    No_argument "a"
 		"local\0"   No_argument "l"
@@ -157,7 +159,7 @@ int rtcwake_main(int argc UNUSED_PARAM, char **argv)
 		bb_error_msg_and_die("%s not enabled for wakeup events", rtcname);
 
 	/* relative or absolute alarm time, normalized to time_t */
-	sys_time = time(0);
+	sys_time = time(NULL);
 	if (sys_time == (time_t)-1)
 		bb_perror_msg_and_die("read system time");
 	rtc_time = rtc_read_time(fd, utc);
